@@ -6,9 +6,11 @@
 Float32Array, Float64Array,  */
 
 describe('angular', function() {
-  var element;
+  var element, document;
 
-  var document = window.document;
+  beforeEach(function() {
+    document = window.document;
+  });
 
   afterEach(function() {
     dealoc(element);
@@ -17,9 +19,12 @@ describe('angular', function() {
   describe('case', function() {
     it('should change case', function() {
       expect(lowercase('ABC90')).toEqual('abc90');
-      expect(manualLowercase('ABC90')).toEqual('abc90');
       expect(uppercase('abc90')).toEqual('ABC90');
-      expect(manualUppercase('abc90')).toEqual('ABC90');
+    });
+
+    it('should change case of non-ASCII letters', function() {
+      expect(lowercase('Ω')).toEqual('ω');
+      expect(uppercase('ω')).toEqual('Ω');
     });
   });
 
@@ -600,6 +605,31 @@ describe('angular', function() {
       expect(copy(new Number(NaN)).valueOf()).toBeNaN();
       /* eslint-enable */
     });
+
+    it('should copy source until reaching a given max depth', function() {
+      var source = {a1: 1, b1: {b2: {b3: 1}}, c1: [1, {c2: 1}], d1: {d2: 1}};
+      var dest;
+
+      dest =  copy(source, {}, 1);
+      expect(dest).toEqual({a1:1, b1:'...', c1:'...', d1:'...'});
+
+      dest =  copy(source, {}, 2);
+      expect(dest).toEqual({a1:1, b1:{b2:'...'}, c1:[1,'...'], d1:{d2:1}});
+
+      dest =  copy(source, {}, 3);
+      expect(dest).toEqual({a1: 1, b1: {b2: {b3: 1}}, c1: [1, {c2: 1}], d1: {d2: 1}});
+
+      dest =  copy(source, {}, 4);
+      expect(dest).toEqual({a1: 1, b1: {b2: {b3: 1}}, c1: [1, {c2: 1}], d1: {d2: 1}});
+    });
+
+    they('should copy source and ignore max depth when maxDepth = $prop',
+      [NaN, null, undefined, true, false, -1, 0], function(maxDepth) {
+        var source = {a1: 1, b1: {b2: {b3: 1}}, c1: [1, {c2: 1}], d1: {d2: 1}};
+        var dest =  copy(source, {}, maxDepth);
+        expect(dest).toEqual({a1: 1, b1: {b2: {b3: 1}}, c1: [1, {c2: 1}], d1: {d2: 1}});
+      }
+    );
   });
 
   describe('extend', function() {
@@ -784,6 +814,19 @@ describe('angular', function() {
       expect(isElement(dst.jqObject)).toBeTruthy();
       expect(dst.jqObject.nodeName).toBeUndefined(); // i.e it is a jqLite/jQuery object
     });
+
+    it('should not merge the __proto__ property', function() {
+      var src = JSON.parse('{ "__proto__": { "xxx": "polluted" } }');
+      var dst = {};
+
+      merge(dst, src);
+
+      if (typeof dst.__proto__ !== 'undefined') { // eslint-disable-line
+        // Should not overwrite the __proto__ property or pollute the Object prototype
+        expect(dst.__proto__).toBe(Object.prototype); // eslint-disable-line
+      }
+      expect(({}).xxx).toBeUndefined();
+    });
   });
 
 
@@ -1008,10 +1051,13 @@ describe('angular', function() {
 
     }
 
-    var originalFunction;
+    var originalPrototype = window.Function.prototype;
 
     beforeEach(function() {
       spyOn(window, 'Function');
+      // Jasmine 2.7+ doesn't support spying on Function, so we have restore the prototype
+      // as Jasmine will use Function internally
+      window.Function.prototype = originalPrototype;
     });
 
     afterEach(function() {
@@ -1224,6 +1270,37 @@ describe('angular', function() {
     });
   });
 
+  describe('isArray', function() {
+
+    it('should return true if passed an `Array`', function() {
+      expect(isArray([])).toBe(true);
+    });
+
+    it('should return true if passed an `Array` from a different window context', function() {
+      var iframe = document.createElement('iframe');
+      document.body.appendChild(iframe);  // No `contentWindow` if not attached to the DOM.
+      var arr = new iframe.contentWindow.Array();
+      document.body.removeChild(iframe);  // Clean up.
+
+      expect(arr instanceof Array).toBe(false);
+      expect(isArray(arr)).toBe(true);
+    });
+
+    it('should return true if passed an object prototypically inherited from `Array`', function() {
+      function FooArray() {}
+      FooArray.prototype = [];
+
+      expect(isArray(new FooArray())).toBe(true);
+    });
+
+    it('should return false if passed non-array objects', function() {
+      expect(isArray(document.body.childNodes)).toBe(false);
+      expect(isArray({length: 0})).toBe(false);
+      expect(isArray({length: 2, 0: 'one', 1: 'two'})).toBe(false);
+    });
+
+  });
+
   describe('isArrayLike', function() {
 
     it('should return false if passed a number', function() {
@@ -1351,22 +1428,6 @@ describe('angular', function() {
       expect(log).toEqual(['0:a', '1:c']);
     });
 
-    if (document.querySelectorAll) {
-      it('should handle the result of querySelectorAll in IE8 as it has no hasOwnProperty function', function() {
-        document.body.innerHTML = '<p>' +
-          '<a name=\'x\'>a</a>' +
-          '<a name=\'y\'>b</a>' +
-          '<a name=\'x\'>c</a>' +
-          '</p>';
-
-        var htmlCollection = document.querySelectorAll('[name="x"]'),
-          log = [];
-
-        forEach(htmlCollection, function(value, key) { log.push(key + ':' + value.innerHTML); });
-        expect(log).toEqual(['0:a', '1:c']);
-      });
-    }
-
     it('should handle arguments objects like arrays', function() {
       var args,
           log = [];
@@ -1697,64 +1758,108 @@ describe('angular', function() {
       dealoc(appElement);
     });
 
-    it('should bootstrap from an extension into an extension document for same-origin documents only', function() {
-      if (msie) return;  // IE does not support document.currentScript (nor extensions with protocol), so skip test.
+    // Support: IE 9-11 only
+    // IE does not support `document.currentScript` (nor extensions with protocol), so skip tests.
+    if (!msie) {
+      describe('auto bootstrap restrictions', function() {
 
-      // Extension URLs are browser-specific, so we must choose a scheme that is supported by the browser to make
-      // sure that the URL is properly parsed.
-      var extensionScheme;
-      var userAgent = window.navigator.userAgent;
-      if (/Firefox\//.test(userAgent)) {
-        extensionScheme = 'moz-extension';
-      } else if (/Edge\//.test(userAgent)) {
-        extensionScheme = 'ms-browser-extension';
-      } else if (/Chrome\//.test(userAgent)) {
-        extensionScheme = 'chrome-extension';
-      } else if (/Safari\//.test(userAgent)) {
-        extensionScheme = 'safari-extension';
-      } else {
-        extensionScheme = 'browserext';  // Upcoming standard scheme.
-      }
+        function createFakeDoc(attrs, protocol, currentScript) {
 
-      var src = extensionScheme + '://something';
-      // Fake a minimal document object (the actual document.currentScript is readonly).
-      var fakeDoc = {
-        currentScript: { getAttribute: function() { return src; } },
-        location: {protocol: extensionScheme + ':', origin: extensionScheme + '://something'},
-        createElement: document.createElement.bind(document)
-      };
-      expect(allowAutoBootstrap(fakeDoc)).toBe(true);
+          protocol = protocol || 'http:';
+          var origin = protocol + '//something';
 
-      src = extensionScheme + '://something-else';
-      expect(allowAutoBootstrap(fakeDoc)).toBe(false);
-    });
+          if (currentScript === undefined) {
+            currentScript = document.createElement('script');
+            Object.keys(attrs).forEach(function(key) { currentScript.setAttribute(key, attrs[key]); });
+          }
 
-    it('should not bootstrap from an extension into a non-extension document', function() {
-      if (msie) return;  // IE does not support document.currentScript (nor extensions with protocol), so skip test.
+          // Fake a minimal document object (the actual document.currentScript is readonly).
+          return {
+            currentScript: currentScript,
+            location: {protocol: protocol, origin: origin},
+            createElement: document.createElement.bind(document)
+          };
+        }
 
-      var src = 'resource://something';
-      // Fake a minimal document object (the actual document.currentScript is readonly).
-      var fakeDoc = {
-        currentScript: { getAttribute: function() { return src; } },
-        location: {protocol: 'http:'},
-        createElement: document.createElement.bind(document)
-      };
-      expect(allowAutoBootstrap(fakeDoc)).toBe(false);
+        describe('from extensions into extension documents', function() {
+          // Extension URLs are browser-specific, so we must choose a scheme that is supported by the browser to make
+          // sure that the URL is properly parsed.
+          var protocol;
+          var userAgent = window.navigator.userAgent;
+          if (/Firefox\//.test(userAgent)) {
+            protocol = 'moz-extension:';
+          } else if (/Edge\//.test(userAgent)) {
+            protocol = 'ms-browser-extension:';
+          } else if (/Chrome\//.test(userAgent)) {
+            protocol = 'chrome-extension:';
+          } else if (/Safari\//.test(userAgent)) {
+            protocol = 'safari-extension:';
+          } else {
+            protocol = 'browserext:';  // Upcoming standard scheme.
+          }
 
-      src = 'file://whatever';
-      expect(allowAutoBootstrap(fakeDoc)).toBe(true);
-    });
+          it('should bootstrap for same-origin documents', function() {
+            expect(allowAutoBootstrap(createFakeDoc({src: protocol + '//something'}, protocol))).toBe(true);
+          });
 
-    it('should not bootstrap if bootstrapping is disabled', function() {
-      isAutoBootstrapAllowed = false;
-      angularInit(jqLite('<div ng-app></div>')[0], bootstrapSpy);
-      expect(bootstrapSpy).not.toHaveBeenCalled();
-      isAutoBootstrapAllowed = true;
-    });
+          it('should not bootstrap for cross-origin documents', function() {
+            expect(allowAutoBootstrap(createFakeDoc({src: protocol + '//something-else'}, protocol))).toBe(false);
+          });
+
+        });
+
+
+        it('should bootstrap from a script with no source (e.g. src, href or xlink:href attributes)', function() {
+
+          expect(allowAutoBootstrap(createFakeDoc({src: null}))).toBe(true);
+          expect(allowAutoBootstrap(createFakeDoc({href: null}))).toBe(true);
+          expect(allowAutoBootstrap(createFakeDoc({'xlink:href': null}))).toBe(true);
+        });
+
+        it('should not bootstrap from a script with an empty source (e.g. `src=""`)', function() {
+          expect(allowAutoBootstrap(createFakeDoc({src: ''}))).toBe(false);
+          expect(allowAutoBootstrap(createFakeDoc({href: ''}))).toBe(false);
+          expect(allowAutoBootstrap(createFakeDoc({'xlink:href': ''}))).toBe(false);
+        });
+
+
+        it('should not bootstrap from an extension into a non-extension document', function() {
+
+          expect(allowAutoBootstrap(createFakeDoc({src: 'resource://something'}))).toBe(false);
+          expect(allowAutoBootstrap(createFakeDoc({src: 'file://whatever'}))).toBe(true);
+        });
+
+        it('should not bootstrap from an extension into a non-extension document, via SVG script', function() {
+
+          // SVG script tags don't use the `src` attribute to load their source.
+          // Instead they use `href` or the deprecated `xlink:href` attributes.
+
+          expect(allowAutoBootstrap(createFakeDoc({href: 'resource://something'}))).toBe(false);
+          expect(allowAutoBootstrap(createFakeDoc({'xlink:href': 'resource://something'}))).toBe(false);
+
+          expect(allowAutoBootstrap(createFakeDoc({src: 'http://something', href: 'resource://something'}))).toBe(false);
+          expect(allowAutoBootstrap(createFakeDoc({href: 'http://something', 'xlink:href': 'resource://something'}))).toBe(false);
+          expect(allowAutoBootstrap(createFakeDoc({src: 'resource://something', href: 'http://something', 'xlink:href': 'http://something'}))).toBe(false);
+        });
+
+        it('should not bootstrap if the currentScript property has been clobbered', function() {
+
+          var img = document.createElement('img');
+          img.setAttribute('src', '');
+          expect(allowAutoBootstrap(createFakeDoc({}, 'http:', img))).toBe(false);
+        });
+
+        it('should not bootstrap if bootstrapping is disabled', function() {
+          isAutoBootstrapAllowed = false;
+          angularInit(jqLite('<div ng-app></div>')[0], bootstrapSpy);
+          expect(bootstrapSpy).not.toHaveBeenCalled();
+          isAutoBootstrapAllowed = true;
+        });
+      });
+    }
   });
 
-
-  describe('angular service', function() {
+  describe('AngularJS service', function() {
     it('should override services', function() {
       module(function($provide) {
         $provide.value('fake', 'old');
@@ -1788,6 +1893,43 @@ describe('angular', function() {
     });
   });
 
+  describe('isError', function() {
+    function testErrorFromDifferentContext(createError) {
+      var iframe = document.createElement('iframe');
+      document.body.appendChild(iframe);
+      try {
+        var error = createError(iframe.contentWindow);
+        expect(isError(error)).toBe(true);
+      } finally {
+        iframe.parentElement.removeChild(iframe);
+      }
+    }
+
+    it('should not assume objects are errors', function() {
+      var fakeError = { message: 'A fake error', stack: 'no stack here'};
+      expect(isError(fakeError)).toBe(false);
+    });
+
+    it('should detect simple error instances', function() {
+      expect(isError(new Error())).toBe(true);
+    });
+
+    it('should detect errors from another context', function() {
+      testErrorFromDifferentContext(function(win) {
+        return new win.Error();
+      });
+    });
+
+    it('should detect DOMException errors from another context', function() {
+      testErrorFromDifferentContext(function(win) {
+        try {
+          win.document.querySelectorAll('');
+        } catch (e) {
+          return e;
+        }
+      });
+    });
+  });
 
   describe('isRegExp', function() {
     it('should return true for RegExp object', function() {
